# This Praat script extracts pitch, intensity, three formants and corresponding formant bandwidths for every 10ms
# This code is a lot based on the one written by Setsuko Shirai
# And a little modified (formants and their bandwidths extraction parts were added) by Jangwon Kim
# Signal Analysis and Interpretation Lab (SAIL)
# University of Southern California
# Electrical Engineering Ph.D. candidate
# Jangwon Kim
# jangwon@usc.edu
# July 6th 2010
# If you found errors or have questions, advices, comments, etc., please do not hesitate and let me know via email.
# Change "PATH"

# Parameter setting section-----------------------
type_file$ = "wav"
# below line can be used for 'To LPC + To Pitch' functions (simple parameter setting)
# prediction_order = 11

# Pitch (ac) parameters (detailed parameter setting)
# male: 75-300, female: 100-600 in my database.
minimum_pitch = 75
maximum_pitch = 350
max_num_candidate = 15
sil_thld = 0.04
vo_thld = 0.45
octv_cost = 0.01
octv_jump_cost = 0.70
v_uv_cost = 0.18

win_length = 0.025
win_shift = 0.005

# When you use To Formant (burg)
max_num_formants = 5
max_formant = 5000

# When you use To LPC (autocorrelation) -> To Formant
# new_sample_rate = 11000
# ---------------------------------

form supply_arguments
	sentence input_directory  "PATH"\WavFile
	sentence output_directory "PATH"\out
endform

# finding files we are looking for
Create Strings as file list... list 'input_directory$'\*.'type_file$'
# the name of files - later we could track each file
numberOfFiles = Get number of strings
for ifile to numberOfFiles
	select Strings list
	fileName$ = Get string... ifile
	Read from file... 'input_directory$'\'fileName$'
endfor

select all
numSelected = numberOfSelected ("Sound")

# change the name of each file - for batch processing
for i to numSelected
	select all
	currSoundID = selected ("Sound", i)
	select 'currSoundID'
	currName$ = "word_'i'"
	Rename... 'currName$'
endfor

for i to numSelected 
	select Sound word_'i'

	# get the finishing time of the Sound file
	fTime = Get finishing time
	fTime = floor(fTime * 100) / 100

      	# Use numTimes in the loop
	numTimes = fTime / win_shift
	newName$ = "word_'i'"
	select Sound word_'i'

      	# 1st argument: New sample rate 2nd argument: Precision (samples)
	# Resample... 'new_sample_rate' 50 apply this only when you use To LPC for formant extraction

	# 1st argument: Time step (s), 2nd argument: Minimum pitch for Analysis, 
	# 3rd argument: Maximum pitch for Analysis
	# (1) When Simple pitch extraction
	# To Pitch... 'win_shift' 'minimum_pitch' 'maximum_pitch'
	# (2) AC pitch extraction: The full names of its arguments are following: 
	# time_step min_f0 max_#candidate, Very accurate, Silence Threshold, Voicing Threshold
	# Octave cost, Octave-jump cost, Voiced/unvoiced cost, max_f0
	To Pitch (ac)... 'win_shift' 'minimum_pitch' 'max_num_candidate' no 'sil_thld' 'vo_thld' 'octv_cost' 'octv_jump_cost' 'v_uv_cost' 'maximum_pitch'

	Rename... 'newName$'

	# select Sound word_'i'_'new_sample_rate'
	select Sound word_'i'
	To Intensity... 'minimum_pitch' 0
	
	# select Sound word_'i'_'new_sample_rate'
	select Sound word_'i'
	# (1) Formant extraction by LPC (AC)
	# 1st argument:  prediction order, 2nd argument: Analysis width (seconds)
	# 3rd argument: Time step (seconds),  4th argument: Pre-emphasis from (Hz)
	# To LPC (autocorrelation)... 'prediction_order'  'win_length' 'win_shift' 50
	# To Formant
	# (2) Formant extraction by To Formant (burg)
	To Formant (burg)... 'win_shift' 'max_num_formants' 'max_formant' 'win_length' 50

	Create Table... table_word_'i' numTimes 9
	Set column label (index)... 1 T
	Set column label (index)... 2 F0
	Set column label (index)... 3 E
	Set column label (index)... 4 F1
	Set column label (index)... 5 FB1
	Set column label (index)... 6 F2
	Set column label (index)... 7 FB2
	Set column label (index)... 8 F3
	Set column label (index)... 9 FB3

	for itime to numTimes
		select Pitch word_'i'
		curtime = win_shift * itime

		f0 = 0
		f0 = Get value at time... 'curtime' Hertz Linear
		f0$ = fixed$ (f0, 2)

		if f0$ = "--undefined--"
			f0$ = "0"
		endif

		curtime$ = fixed$ (curtime, 5)
		 
		# select Intensity word_'i'_'new_sample_rate'
		select Intensity word_'i'
		intensity = Get value at time... 'curtime' Cubic
		
		
		intensity$ = fixed$ (intensity, 2)
		if intensity$ = "--undefined--"
			intensity$ = "0"
		endif

		# select Formant word_'i'_'new_sample_rate'
		select Formant word_'i'

		f1 = Get value at time... 1 'curtime' Hertz Linear
		f1$ = fixed$ (f1, 2)
		if f1$ = "--undefined--"
			f1$ = "0"
		endif

		fb1 = Get bandwidth at time... 1 'curtime' Hertz Linear
		fb1$ = fixed$ (fb1, 2)
		if fb1$ = "--undefined--"
			fb1$ = "0"
		endif
		
		f2 = Get value at time... 2 'curtime' Hertz Linear
		f2$ = fixed$ (f2, 2)
		if f2$ = "--undefined--"
			f2$ = "0"
		endif

		fb2 = Get bandwidth at time... 2 'curtime' Hertz Linear
		fb2$ = fixed$ (fb2, 2)
		if fb2$ = "--undefined--"
			fb2$ = "0"
		endif

		f3 = Get value at time... 3 'curtime' Hertz Linear
		f3$ = fixed$ (f3, 2)
		if f3$ = "--undefined--"
			f3$ = "0"
		endif

		fb3 = Get bandwidth at time... 3 'curtime' Hertz Linear
		fb3$ = fixed$ (fb3, 2)
		if fb3$ = "--undefined--"
			fb3$ = "0"
		endif

		select Table table_word_'i'
		Set numeric value... itime T 'curtime$'
		Set numeric value... itime F0 'f0$' 
		Set numeric value... itime E  'intensity$'
		Set numeric value... itime F1 'f1$'
		Set numeric value... itime FB1 'fb1$'
		Set numeric value... itime F2 'f2$' 
		Set numeric value... itime FB2 'fb2$'
		Set numeric value... itime F3 'f3$' 
		Set numeric value... itime FB3 'fb3$'
	endfor
	select Strings list
	fileName$ = Get string... i
	select Table table_word_'i'
	Write to table file... 'output_directory$'\'fileName$'.txt
endfor
